
/*
 * This file implements initial version of device-mapper dklinear target.
 */

#include <sys/types.h>
#include <sys/param.h>

#include <sys/buf.h>
#include <sys/disk.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <sys/ioccom.h>
#include <sys/kmem.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/queue.h>
#include <sys/vnode.h>

#include "dm.h"

static int atoi(char *);

/*
 * Allocate target specific config data, and link them to table.
 * argv[0] is name,
 * argv[1] is physical data offset.
 */
int
dm_target_linear_init(struct dm_dev *dmv, void **target_config, int argc, char **argv)
{
	struct target_linear_config *tlc;
	struct dm_pdev *dmp;
		
	printf("Linear target init function called %s--%s!!\n",argv[0],argv[1]);
	
	dmp = dm_pdev_lookup_name(argv[0]);
		
	if ((tlc = kmem_alloc(sizeof(struct target_linear_config),KM_NOSLEEP))
	    == NULL)
		return 1;

	// XXX howto convert char* to number ? tlc->offset = atoi(argv[1]);

	tlc->pdev = dmp;
	tlc->offset = atoi(argv[1]);
	printf("offset = %d\n",tlc->offset);
	*target_config = tlc;    
	
	return 0;
}

/*
 * Do IO operation, called from dmstrategy routine.
 */

int
dm_target_linear_strategy(struct dm_table_entry *table_en, struct buf *bp)
{
	struct target_linear_config *tlc;

	tlc = table_en->target_config;
	
	printf("Linear target read function called!!\n");

	bp->b_blkno += tlc->offset;
	
	VOP_STRATEGY(tlc->pdev->pdev_vnode, bp);
	
	return 0;

}

/*
 * Destroy target specific data.
 */

int
dm_target_linear_destroy(struct dm_table_entry *table_en)
{

	kmem_free(table_en->target_config,sizeof(struct target_linear_config));
	
	return 0;
}

/*
 * Register upcall device.
 */

int
dm_target_linear_upcall(struct dm_table_entry *table_en, struct buf *bp)
{

	return 0;
}

static int
atoi(char *s)
{
	int n, neg;

	n = 0;
	neg = 0;

	while (*s == '-') {
		s++;
		neg = !neg;
	}

	while (*s != '\0') {
		if (*s < '0' && *s > '9')
			break;

		n = (10 * n) + (*s - '0');
		s++;
	}

	return (neg ? -n : n);
}
